////////////////////////////////////////////////////////////////////////////////
//
//		Title		: NSlider.h File
//
//		Copyright (C) 2006, NHN Co., Ltd. ALL RIGHTS RESERVED
//
//		Team		: Contents Manager Development
//
//		author		: hhd2002
//
//		created		: 14:2:2007   16:35
//  
//////////////////////////////////////////////////////////////////////////////// 

#pragma once

#include "ImageProxy.h"
#include "NControl.h"
#include "NPCommonUI.h"
#include "NSliderThumb.h"
#include "NToolTip.h"
#include "Panel.h"
#include "StateControl.h"
#include "ScriptletContainerFactory.h"

class CKeyListener_Slider;
interface INSliderListener;
interface INSliderUI;

/**
 * \group NPCommonUI 
 * \date 2007-02-14
 * \author hhd
 * \brief {kr} slider class {/kr}
 * \remark 
	{kr} 
	  ̴ ؼ ó ߵ. 
	÷ Ŵ ̴ ؼ . (070611)
	{/kr} 
 */
class NPCOMMONUI_API CNSlider	
	: public CNPanel
	, public ILayoutManager
{
	DECLARE_RTTI
	DECLARE_COMPONENT_CREATOR(CNSlider);

	friend class CMouseListener_Slider;
	friend class CMouseListener_Slider_Thumb;
	friend class CSliderDefUI;
	friend class CSliderThumbLayoutManager;

// enum
public:
	///< {kr} ̴  {/kr}
	enum eDirection
	{
		E_SLIDER_DIRECTION_HORZ, ///< {kr} ι ̴. ⺻ {/kr}
		E_SLIDER_DIRECTION_VERT,
	};

	///< {kr} ̴  setϴ Ӽ {/kr}
	enum eProperty
	{
		E_SLIDER_PROP_MIN,
		E_SLIDER_PROP_MAX,
		E_SLIDER_PROP_POS,
	};

	/**
	 * \param eDir {kr} slider  {/kr}
	 */
	CNSlider(eDirection eDir = E_SLIDER_DIRECTION_HORZ);
	virtual ~CNSlider();

	VBEGIN_MSG_MAP(CNSlider)
		MESSAGE_HANDLER(WM_NCACTIVATE, OnNcActivate)

		CHAIN_MSG_MAP(CNPanel)
	END_MSG_MAP()

	DECLARE_MESSAGE_HANDLER(OnNcActivate);

	//virtual
	/**
	* \param nMin {kr} slider nMin {/kr}
	*/
	virtual BOOL SetMinValue(INT64 nMin);
	/**
	* \param nMin {kr} slider nMax {/kr}
	*/
	virtual BOOL SetMaxValue(INT64 nMax);

	/**
	* \return {kr} slider nMin {/kr}
	*/
	INT64 GetMinValue();
	/**
	* \remark {kr} slider nMax {/kr}
	*/
	INT64 GetMaxValue();
	/**
	* \return {kr} slider nPos {/kr}
	*/
	INT64 GetPosValue();
	/**
	* \brief {kr} logical pos . {/kr}
	* \param nPos {kr} logical pos {/kr}
	* \param bUpdate {kr} pos  ´ UI Ʈ  ΰ? {/kr}
	* \param bNotify {kr}  SliderListner notify  ΰ? {/kr}
	*/
	void SetPosValue(INT64 nPos);

	__declspec(deprecated) // {kr} void SetPosValue(int nPos);  . {/kr}
	void SetPosValue(INT64 nPos, BOOL bUpdate, BOOL bNotify);
	/**
	 * \return {kr} slider  {/kr}
	 */
	eDirection GetDirection();
	/**
	 * \param eDir {kr} slider  {/kr}
	 */
	void SetDirection(eDirection eDir);
	/**
	 * \breif {kr} slider event  ü Ѵ. {/kr}
	 */
	void AddSliderListener(INSliderListener* pSliderListener);
	void RemoveSliderListener(INSliderListener* pSliderListener);
	
	/**
	 * \brief {kr} ؽƮ . {/kr}
	 * \param szToolTipText 
	 */
	void SetToolTipText(LPCTSTR szToolTipText);
	/**
	* \brief {kr} ̴ Ϻκ RECT Ѵ.{/kr}
	*/
	void SetRailRect(LPRECT prcRail);
	void GetRailRect(LPRECT prcRail);
	/**
	* \brief {kr}   RECT. {/kr}
	*/
	void SetThumbStartRect(LPRECT prcThumbStart);
	void GetThumbStartRect(LPRECT prcThumbStart);
	/**
	* \brief {kr}   RECT.{/kr}
	*/
	void SetThumbEndRect(LPRECT prcThumbEnd);
	void GetThumbEndRect(LPRECT prcThumbEnd);
	/**
	 * \brief {kr} ѹ keydown ̴ Ÿ . {/kr}
	 * \param nKeyHop 
	 */
	void SetKeyHop(INT64 nKeyHop);
	/**
	 * \brief {kr} ѹ keydown ̴ Ÿ . {/kr}
	 * \return 
	 */
	INT64 GetKeyHop();
	/**
	 * \brief {kr} keydown ߻ϴ ֱ⸦ . {/kr}
	 * \param nKeyPressedTimePeriod 
	 */
	void SetKeyPressedTimePeriod(int nKeyPressedTimePeriod);
	/**
	 * \brief {kr} keydown ߻ϴ ֱ⸦ . {/kr}
	 * \return 
	 */
	int GetKeyPressedTimePeriod();
	/**
	 * \brief {kr} Ʈ ͸ . {/kr}
	 * \return {kr} Ʈ . {/kr}
	 */
	CNToolTip* GetToolTip();
	/**
	* \brief {kr} ̴ ͸ . {/kr}
	* \return {kr} ̴ . {/kr}
	*/
	CNSliderThumb* GetThumb();

	// override
	LPCTSTR GetClassName() { return SCRIPTLET_SLIDER_NODE; }
	// CContainer
	void DrawComponent(HDC hDC);
	// CComponent
	void OnCreate();
	void OnDestroy();
	void OnMouseLeave();
	void SetUI(IComponentUI* pComponentUI);
	BOOL EnableWindow(BOOL bEnabled);
	// ILayoutManager
	void LayoutContainer(CContainer* pContainer);

protected:
	// PM-714 2007/10/23 21:18:12 
	/* {kr}
		Update_Logic2Device, Update_Device2Logic߰.
		̴  ˰   ó  ֵ .
		ŸƲ/  ߰ ̴  䱸 Ǵ Ϳ,
		override Ͽ ϰ   ִ  .
	{/kr} */
	/**
	 * \brief {kr} thumb ġ ̽ġ ݿ. {/kr}
	 */
	virtual void Update_Logic2Device();
	/**
	 * \brief {kr} thumb ̽ġ ġ ݿ. {/kr}
	 */
	virtual void Update_Device2Logic(BOOL bNotify = TRUE);
	// {kr} thumb г. Ŀ óϱ  Ŭ. {/kr}
	CNSliderThumb m_Panel_Thumb;

private:
	void _Fire_OnSliderPressed();
	void _Fire_OnSliderDragging();
	void _Fire_OnSliderReleased(BOOL bCancel);
	void _Fire_OnSliderPropertyChanged(eProperty eProp);

	BOOL m_bPressed;
	
	CAtlList<INSliderListener*> m_vSliderListener;
	
	CAutoPtr<CKeyListener_Slider> m_pKeyListener_Slider;
	CAutoPtr<CMouseListener_Slider>	m_pMouseListner_Slider;
	CAutoPtr<CMouseListener_Slider_Thumb>	m_pMouseListner_Slider_Thumb;
	
	CNToolTip m_ToolTip;
	
	CRect m_rcRail;
	CRect m_rcThumbEnd;
	CRect m_rcThumbStart;
	
	CStateControl m_StatePanelSlider;
	CStateControl m_StatePanelThumb;
	
	eDirection m_eDirection;
	
	INT64 m_nKeyHop;
	INT64 m_nMax;
	INT64 m_nMin;
	INT64 m_nPos;
	int m_nKeyPressedTimePeriod;

	static const int Prop_ToolTipOffsetX = 0;
	static const int Prop_ToolTipOffsetY = 20;
	static const INT64 NSLIDER_POS_INVALID;
};

//////////////////////////////////////////////////////////////////////////

/**
 * \group NPCommonUI 
 * \date 2007-02-14
 * \author hhd
 * \brief {kr} ̴ ̺Ʈ  listener. ַ ø̼ ȴ. {/kr}
 */
interface NPCOMMONUI_API INSliderListener
{
public:
	virtual ~INSliderListener() {  }
	/**
	 * \breif {kr} ̴ Bg pressed {/kr}
	 */
	virtual void OnSliderPressed() = 0;
	/**
	 * \breif {kr} ̴ Bg 巡. κ pressed up óѴ. dragging óϴ  幰. {/kr}
	 */
	virtual void OnSliderDragging() = 0;
	/**
	 * \breif {kr} ̴ Bg up. {/kr}
	 * \param bCancel {kr} ̴   ҵǾ  TRUE {/kr}
	 */
	virtual void OnSliderReleased(BOOL bCancel) = 0;
	/**
	 * \breif {kr}	̴ Ӽ ɶ. eProp ̴ Ӽ. 
					̴ Ӽ Ǵ  ϸ, 
					̴ ü full model ͼ ؼ ݿŰ  . {/kr}
	 */
	virtual void OnSliderPropertyChanged(CNSlider::eProperty eProp) = 0;
};

//////////////////////////////////////////////////////////////////////////

class CMouseListener_Slider_Thumb;
/**
* \group NPCommonUI 
* \date 2007-02-14
* \author hhd
* \brief {kr} ̴ Bg  콺 listener {/kr}
*/
class CMouseListener_Slider : public IMouseListener
{
public:
	CMouseListener_Slider(
		CNSlider* pSlider, 
		CMouseListener_Slider_Thumb* pMouseListener_Slider_Thumb);

	//overrided
	//IMouseListener
	void OnMousePressed(CMouseEvent* pMouseEvent);
	void OnMouseReleased(CMouseEvent* pMouseEvent);
	void OnMouseEntered(CMouseEvent* pMouseEvent);
	void OnMouseExited(CMouseEvent* pMouseEvent);
	void OnMouseMoved(CMouseEvent* pMouseEvent) {}
	void OnMouseWheelMoved(CMouseEvent* pMouseEvent) {}

	CPoint m_ptPressed;
private:
	CNSlider* m_pSlider;
	CMouseListener_Slider_Thumb* m_pMouseListener_Slider_Thumb;
};

//////////////////////////////////////////////////////////////////////////

/**
* \group NPCommonUI 
* \date 2007-02-14
* \author hhd
* \brief {kr} ̴ thumb 콺 listener {/kr}
*/
class CMouseListener_Slider_Thumb : public IMouseListener
{
public:
	CMouseListener_Slider_Thumb(CNSlider* pSlider, CNPanel* pThumb);

	/**
	 * \breif {kr}	 . 
					   2 Ʈ Ʈ .
					1. ڰ  콺 ŬҶ
					2. ڰ ٵ Ŭ
						  ġ 鼭  Ŭ  ȿ 
					  2  ٸ ƮƮ  ˰   ִ  . {/kr}
	 */
	void DoPressed(LPPOINT pptPressed);
	/**
	* \breif {kr}	   . 
					DoPressed  pair ̷. {/kr}
	*/
	void DoReleased(LPPOINT pptReleased);

	//overrided
	//IMouseListener
	void OnMousePressed(CMouseEvent* pMouseEvent);
	void OnMouseReleased(CMouseEvent* pMouseEvent);
	void OnMouseEntered(CMouseEvent* pMouseEvent);
	void OnMouseExited(CMouseEvent* pMouseEvent);
	void OnMouseMoved(CMouseEvent* pMouseEvent);
	void OnMouseWheelMoved(CMouseEvent* pMouseEvent) {}

private:
	CNPanel* m_pThumb;
	CNSlider* m_pSlider;
	CPoint m_ptOld;
	CRect m_rcBoundPressed;
	BOOL m_bEnter;
};

//////////////////////////////////////////////////////////////////////////

/**
* \group NPCommonUI 
* \date 2007-02-14
* \author hhd
* \brief {kr} ̴ UI {/kr}
*/
interface NPCOMMONUI_API INSliderUI : public IComponentUI
{
	DECLARE_RTTI
public:
	virtual ~INSliderUI() {  }

	/**
	 * \brief {kr} ̴ ׸ Լ. {/kr}
	 * \param hDC {kr} ̴ DC {/kr}
	 * \param prcSlider {kr} ̴ . {/kr}
	 * \param prcThumb {kr} thumb {/kr}
	 * \param eSliderState {kr} ̴  . {/kr}
	 * \param eThumbState {kr} thumb  . {/kr}
	 */
	virtual void DrawSlider(HDC hDC, 
							LPRECT prcSlider, 
							LPRECT prcThumb, 
							CStateControl::eControlState eSliderState, 
							CStateControl::eControlState eThumbState) = 0;
};

//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
// PM-566 2007/10/05 11:35:26 
//////////////////////////////////////////////////////////////////////////
class CKeyListener_Slider : public IKeyListener
{
public:
	CKeyListener_Slider(CNSlider* pSlider);
	virtual ~CKeyListener_Slider();

	//override
	//IKeyListener
	void	OnKeyPressed(CKeyEvent* pKeyEvent);
	void	OnKeyReleased(CKeyEvent* pKeyEvent);
protected:
private:
	CNSlider* m_pSlider;
};